local assignSoldier = ...

--------------------------------------------------------------------------
local ASSIGN_SOLDIER_FIGHT_COMP = {
	TypeName = "ASSIGN_SOLDIER_FIGHT_COMP",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "Ally", Type = "string", Default = "Ally" },
		{ Name = "SwordJob", Type = "string", Default = "" },
		{ Name = "SpearJob", Type = "string", Default = "" },
		{ Name = "PikeJob", Type = "string", Default = "" },
		{ Name = "HandJob", Type = "string", Default = "" }
	}
}

--------------------------------------------------------------------------
function ASSIGN_SOLDIER_FIGHT_COMP:memorizeAgent(_agent)
--    assignSoldier:log("AssignSoldier: ASSIGN_SOLDIER_FIGHT_COMP:memorizeAgent" .. tostring(_agent:getOwner()))
    self.soldierAssignAgent = _agent
end

--------------------------------------------------------------------------
function ASSIGN_SOLDIER_FIGHT_COMP:init()
--    assignSoldier:log("AssignSoldier: ASSIGN_SOLDIER_FIGHT_COMP:init")
    self.fighting = true
end

--------------------------------------------------------------------------
function ASSIGN_SOLDIER_FIGHT_COMP:computeProximity(_comp, _pos1)
                    local pos2 = _comp:getOwner():getGlobalPosition()
                    local distance = (_pos1.x - pos2.x)^2 + (_pos1.z - pos2.z)^2
                    return distance
end

--------------------------------------------------------------------------
function ASSIGN_SOLDIER_FIGHT_COMP:update()
    if (self.soldierAssignAgent == nil) then
        return
    end
    if (self:computeProximity(self, self.soldierAssignAgent:getOwner():getGlobalPosition()) > 1) then
        return
    end
    local JobName = self.soldierAssignAgent:getOwner():getEnabledComponent("COMP_VILLAGER"):getAssetJob().JobName
--    assignSoldier:log("ASSIGN_SOLDIER_FIGHT_COMP:update() check job soldierAssignAgent " .. tostring(self.soldierAssignAgent:getOwner()) .. " " .. JobName)
    if self.improvedEnnemy == nil then 
        if JobName == self.Ally then
            self.improvedEnnemy = {self.SwordJob, self.SpearJob, self.PikeJob, self.HandJob}
        elseif (JobName == self.SwordJob) or (JobName == self.SpearJob) or (JobName == self.PikeJob) or (JobName == self.HandJob) then
            self.improvedEnnemy = {self.Ally}
        else
            assignSoldier:log("ASSIGN_SOLDIER_FIGHT_COMP:update() check job soldierAssignAgent: " .. JobName .. " not ally " .. self.Ally .. " nor ennemy " .. self.Ennemy)
            return
        end
        assignSoldier:log("ASSIGN_SOLDIER_FIGHT_COMP:update() improvedEnnemy: " .. tostring(self.improvedEnnemy))
    end
    self.Proximity = false
    self:getLevel():getComponentManager("COMP_AGENT"):getAllComponent():forEach(
        function(comp)
            if self.Proximity then
                return --already found
            end
            local _villager = comp:getOwner():getEnabledComponent("COMP_VILLAGER")
            if (_villager ~= nil) and (_villager:getAssetJob() ~= nil) then
                for i,ennemy in ipairs(self.improvedEnnemy) do    
                    if (_villager:getAssetJob().JobName == ennemy) and (self:computeProximity(comp, self.soldierAssignAgent:getOwner():getGlobalPosition()) < 25) then
--                        assignSoldier:log("ASSIGN_SOLDIER_FIGHT_COMP:update() ennemy detected " .. comp:getOwner().Name .. " " .. tostring(_villager:getAssetJob().JobName))
                        self.Target = comp:getOwner():getGlobalPosition()
                        self.Proximity = true
                        return;
                    end
                end
            end
        end
    )
    if self.Proximity then
        self.soldierAssignAgent:getOwner():lookAt(self.Target)
        if not self.fighting then
            if JobName == "SOLDIER" then
                self.soldierAssignAgent:setAnimation("")
            elseif JobName == self.SwordJob then
                self.soldierAssignAgent:setAnimation("SWORD_ATTACK_01")
            elseif JobName == self.SpearJob then
                self.soldierAssignAgent:setAnimation("SPEAR_ATTACK")
            elseif JobName == self.PikeJob then
                self.soldierAssignAgent:setAnimation("SPEAR_ATTACK")
            elseif JobName == self.HandJob then
                self.soldierAssignAgent:setAnimation("HAND_ATTACK")
            else
                self.soldierAssignAgent:setAnimation("")
            end
            self.fighting = true
        end
    else--if self.fighting then
        self.soldierAssignAgent:setAnimation("GUARD")
        self.fighting = false
    end
end

--------------------------------------------------------------------------
assignSoldier:registerClass(ASSIGN_SOLDIER_FIGHT_COMP)


